/*  ***** BEGIN LICENSE BLOCK *****
 *  Version: RPPL 1.0/GPL 2.0/LGPL 2.1/MPL 1.1/CDDL 1.0
 *  
 *  
 *  The contents of this file are subject to the Roland Pihlakas Public
 *  License Version 1.0 (the "License"); you may not use this file except
 *  in compliance with the License. You may obtain a copy of the License
 *  at http://roland.pri.ee/RPPL/
 *  
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *  
 *  The Original Code is: Modelling of Natural Thinking.
 *  
 *  The Initial Developer of the Original Code is: Roland Pihlakas.
 *  
 *  Portions created by: _____________________________ 
 *  are Copyright (C): _____________________________. 
 *  All Rights Reserved.
 *  Contributor(s): ______________________________________.
 *  
 *  Alternatively, the contents of this file may be used under the terms of 
 *  either the GNU General Public License Version 2 or later (the "GPL"), or 
 *  the GNU Lesser General Public License Version 2.1 or later (the "LGPL"), or 
 *  the Mozilla Public Licence Version 1.1 or later (the "MPL"), or the Common 
 *  Development and Distribution License Version 1.0 or later (the "CDDL"), in 
 *  which case the provisions of the MPL, the GPL, the LGPL or the CDDL are 
 *  applicable instead of those above. If you wish to allow use of your version 
 *  of this file only under the terms of the of either the MPL, the GPL, 
 *  the LGPL or the CDDL, and not to allow others to use your version of this 
 *  file under the RPPL, indicate your decision by deleting the provisions 
 *  above and replace them with the notice and other provisions required by 
 *  the MPL, the GPL, the LGPL or the CDDL. If you do not delete the provisions 
 *  above, a recipient may use your version of this file under the terms of any 
 *  one of the RPPL, the MPL, the GPL, the LGPL or the CDDL.
 *
 *  ***** END LICENSE BLOCK ***** */

//######################################################################################################
//
//######################################################################################################





#ifndef CLIENTSERVER_COMMON_H
#define CLIENTSERVER_COMMON_H



//#define _CRTDBG_MAP_ALLOC			   //detect memory leaks
#include <stdlib.h>
#include <crtdbg.h>




#include <stddef.h>				///NULL


///########################################################################
///
///########################################################################

enum SERVER_SIGNALS
{
	SERVER_NOP,
	
	REPLY_TO_SERVER,
	DONE_TO_SERVER,

	SERVER_INIT,
	SERVER_UNINIT,
	SERVER_EXECUTE_STEP,
	
	SERVER_SET_INPUTS,
	SERVER_TICK_ECHO_LINES,
	SERVER_CALC_PREDICTIONS,
	SERVER_GET_OUTPUTS,
	
	SERVER_GET_INNERCELLS,
	
	SERVER_RESET_STATUS_VECTOR,
	SERVER_ACTIVATE_STATUS_VAL,
	SERVER_GET_BLENDED_STATUS_VECTOR,
	SERVER_GET_STATUS_VAL,
	
	SERVER_ONKEYPRESSED,
};

///########################################################################
///
///########################################################################

enum CLIENT_SIGNALS
{
	CLIENT_NOP,
	
	REPLY_TO_CLIENT,
	DONE_TO_CLIENT,

	CLIENT_GET_SENSORPORTID,
	
	CLIENT_GET_SENSORVALUE__BOOL,
	CLIENT_GET_SENSORVALUE__DOUBLE,
	CLIENT_GET_SENSORVALUE__INT,
	CLIENT_GET_SENSORVALUE__DOUBLE_ARRAY,							//TODO
	CLIENT_GET_SENSORVALUE__CHAR_ARRAY,								//TODO
	CLIENT_GET_SENSORVALUE__VECTOR,
	
	CLIENT_GET_SENSORVALUE_FROM_PORTNAME__BOOL,
	CLIENT_GET_SENSORVALUE_FROM_PORTNAME__DOUBLE,
	CLIENT_GET_SENSORVALUE_FROM_PORTNAME__INT,
	CLIENT_GET_SENSORVALUE_FROM_PORTNAME__DOUBLE_ARRAY,				//TODO
	CLIENT_GET_SENSORVALUE_FROM_PORTNAME__CHAR_ARRAY,				//TODO
	CLIENT_GET_SENSORVALUE_FROM_PORTNAME__VECTOR,
	

	CLIENT_GET_STEP,
	
	CLIENT_GET_ACTUATORPORT_ID,
	CLIENT_SET_ACTUATORPORT,	
	CLIENT_SET_ACTUATORPORT_OF_NAME,
	
	CLIENT_DISABLE_PRINTLN,
	CLIENT_ENABLE_PRINTLN,
	CLIENT_PRINTLN,
	CLIENT_PRINT,
	
	CLIENT_SET_CHARTS_DATA,
	
	CLIENT_END_STEP 
};

///########################################################################
///
///########################################################################

typedef struct {

	double dval;
  
	union
	{
		char data[8192];
		int intdata;
	};

} DOUBLE_AND_OTHERDATA;

///########################################################################

typedef struct  
{	
	int header_len;
	int cmd_id;
	int data_len;
}	
REQUEST_STRUCT_HEADER;

///########################################################################

#ifndef __FEACP__			///forward declarations
class Vector3d;
#endif

///########################################################################

class REQUEST_STRUCT 
{
public:
	REQUEST_STRUCT()	{ 
							header.data_len = 0; data = NULL; 
						};
	~REQUEST_STRUCT()	{ 
							if (data != NULL) delete data; 
						};


	REQUEST_STRUCT_HEADER header;

	union
	{
		const void					*voiddata;
		const bool					*booldata;
		const double				*doubledata;
		const Vector3d				*vector3ddata;
		const char					*data;
		const int					*intdata;
		DOUBLE_AND_OTHERDATA		*double_and_otherdata;
		void						*nonconst_void;
	};

};

///########################################################################


#define SEND_STRING(stringval)										\
{																	\
	request.data = stringval.c_str();								\
	request.header.data_len = (int)stringval.size() + 1;			\
}

#define SEND_INT(intval)											\
{																	\
	request.intdata = &intval;										\
	request.header.data_len = sizeof(intval);						\
}

#define SEND_DATA(propname, val)									\
{																	\
	request.##propname = &val;										\
	request.header.data_len = sizeof(val);							\
}
	

#define GET_CLIENTDATA(cmd_id_val, datatype, dataprop, request_init_ops)	\
																		\
																		\
	REQUEST_STRUCT request;												\
	request.header.cmd_id = cmd_id_val;									\
	##request_init_ops;													\
	clientConnection_modal->sendMessage(request);						\
	request.voiddata = NULL;											\
																		\
	REQUEST_STRUCT reply;												\
	int numBytes = clientConnection_modal->recieveMessage(&reply);		\
	if (numBytes == -99)												\
	{																	\
		myException socketOptionException(WSAECONNRESET, "error receiving reply from client");	\
		throw socketOptionException;																\
	};																	\
																		\
	if (reply.header.cmd_id != REPLY_TO_SERVER)							\
	{																	\
		myException socketOptionException(WSAECONNRESET, "wrong reply cmd_id");					\
		throw socketOptionException;																\
	}																	\
																		\
	datatype rval = *(reply.##dataprop);								\
	delete reply.data;													\
	reply.data = NULL;


#define GET_CLIENTDATA_ARRAY(cmd_id_val, datatype, request_init_ops)	\
																		\
																		\
	REQUEST_STRUCT request;												\
	request.header.cmd_id = cmd_id_val;									\
	##request_init_ops;													\
	clientConnection_modal->sendMessage(request);						\
	request.voiddata = NULL;											\
																		\
	REQUEST_STRUCT reply;												\
	int numBytes = clientConnection_modal->recieveMessage(&reply);		\
	if (numBytes == -99)												\
	{																	\
		myException socketOptionException(WSAECONNRESET, "error receiving reply from client");	\
		throw socketOptionException;																\
	};																	\
																		\
	if (reply.header.cmd_id != REPLY_TO_SERVER)							\
	{																	\
		myException socketOptionException(WSAECONNRESET, "wrong reply cmd_id");					\
		throw socketOptionException;																\
	}																	\
																		\
	datatype *rval = (datatype *)reply.nonconst_void;					\
	reply.voiddata = NULL;												\
	unsigned int datapoints = reply.header.data_len / sizeof(datatype);											





#define SEND_CLIENTDATA(cmd_id_val, request_init_ops)					\
{																		\
																		\
	REQUEST_STRUCT request;												\
	request.header.cmd_id = cmd_id_val;									\
	##request_init_ops;													\
	clientConnection_modal->sendMessage(request);						\
	request.voiddata = NULL;											\
};

///########################################################################

#define SEND_ACK(cmd_id_val, connection)								\
{																		\
																		\
	REQUEST_STRUCT ack_out;												\
	ack_out.header.cmd_id = cmd_id_val;									\
	connection->sendMessage(ack_out);									\
	reply_sent = true;													\
};
   
///########################################################################
/// 
///########################################################################

#define send_reply_data(var, myClient)						\
{														    \
	REQUEST_STRUCT mess_struct;							    \
	mess_struct.header.cmd_id = REPLY_TO_SERVER;			\
	mess_struct.voiddata = &var;							\
	mess_struct.header.data_len = sizeof(var);				\
	myConnection->sendMessage(mess_struct);					\
	reply_sent = true;										\
	mess_struct.voiddata = NULL;							\
};	
	

#define send_reply_array(var, myClient, mytype, datalen)	\
{														    \
	REQUEST_STRUCT mess_struct;							    \
	mess_struct.header.cmd_id = REPLY_TO_SERVER;			\
	mess_struct.voiddata = var;								\
	mess_struct.header.data_len = datalen * sizeof(mytype);	\
	myConnection->sendMessage(mess_struct);					\
	reply_sent = true;										\
	mess_struct.voiddata = NULL;							\
};	

	

#define set_request_data(mess_struct, var)		{ mess_struct.data = &var; mess_struct.data_len = sizeof(var); };		

///########################################################################
/// 
///########################################################################


#define GETSENSORVALUE_ARRAY_CASE(suffix, mytype)					\
																	\
	case CLIENT_GET_SENSORVALUE__##suffix##_ARRAY :					\
	{																\
		portID = *mess_in.intdata;									\
		mytype *val;												\
		unsigned int dataunits;										\
		getSensorValue(portID, val, &dataunits);					\
		send_reply_array(val, myClient, mytype, dataunits);			\
	}																\
		break;	
		
#if 0																	
	#define GETSENSORVALUE_ARRAY_CASE_NAME(suffix, mytype)				\
																		\
		case CLIENT_GET_SENSORVALUE_FROM_PORTNAME__##suffix##_ARRAY :	\
		{																\
			int portIDx = getSensorportId(string(mess_in.data));		\											 													 				
			mytype *val;												\
			unsigned int dataunits;										\
			getSensorValue(portIDx, val, &dataunits);					\
			send_reply_array(val, myClient, mytype, dataunits);			\
		}																\
			break;														  

#endif




#define GETSENSORVALUE_CASE(suffix, mytype)							\
																	\
	case CLIENT_GET_SENSORVALUE__##suffix :							\
	{																\
		portID = *mess_in.intdata;									\
		mytype val;													\
		getSensorValue(portID, val);								\
		send_reply_data(val, myClient);								\
	}																\
		break;	
		
#if 0																	
	#define GETSENSORVALUE_CASE_NAME(suffix, mytype)					\
																		\
		case CLIENT_GET_SENSORVALUE_FROM_PORTNAME__##suffix :			\
		{																\
			int portIDx = getSensorportId(string(mess_in.data));		\											 													 				
			mytype val;													\
			getSensorValue(portIDx, val);								\
			send_reply_data(val, myClient);								\
		}																\
			break;														  
		
#endif




#endif	//#ifndef CLIENTSERVER_COMMON_H
